// $Id: mm_midi.c 331 2008-05-10 20:31:33Z tk $
/*
 * MIDIbox MM V2 (C version)
 *
 * ==========================================================================
 *
 *  Copyright (C) 2005 Thorsten Klose (tk@midibox.org)
 *  Licensed for personal non-commercial use only.
 *  All other rights reserved.
 * 
 * ==========================================================================
 */

/////////////////////////////////////////////////////////////////////////////
// Include files
/////////////////////////////////////////////////////////////////////////////

#include <cmios.h>

#include "main.h"
#include <iic_dmx.h>
#include "mdmx_midi.h"
#include "mdmx_dmx.h"
#include "mdmx_lcd.h"
//#include "mmdmx_mf.h"
//#include "mmdmx_dio.h"

/////////////////////////////////////////////////////////////////////////////
// Global Variables
/////////////////////////////////////////////////////////////////////////////
unsigned char dmx_channels[0xff];
unsigned char crc; // This stores the current CRC8 value


unsigned char MDMX_DMX_Update_Channel(unsigned char command, unsigned char channel, unsigned char value) 
{
	unsigned char retry_ctr,i,num_bytes,addr;
	if (dmx_channels[channel]==value)
		return 0;
	crc=0;
	command+=0xb0; // Set command to Control Change 
	MDMX_DMX_CRC8(command);
	MDMX_DMX_CRC8(channel);
	MDMX_DMX_CRC8(value);
//	IIC_DMX_Transmit(0x10, command, channel, value, crc))
	
	if (IIC_DMX_TransmitStart(0x10))
	
	{
		IIC_DMX_TransmitByte(command);
		IIC_DMX_TransmitByte(channel);
		IIC_DMX_TransmitByte(value);
		IIC_DMX_TransmitByte(crc);
		if (channel<0x8)
		{
			MDMX_DMX_ChannelSet(channel,value);
			MDMX_LCD_PointerPrint(channel);
		}
		dmx_channels[channel]=value;	
		IIC_DMX_TransmitStop();
	}
	return 1;
}


void MDMX_DMX_Update_Master(unsigned char level)
{
	unsigned char f,i;
	long channel_level;
	// Send Updates on ALL channels as a percentage of their current value
	if (IIC_DMX_TransmitStart(0x10))
	
	{
		crc=0;
		MDMX_DMX_CRC8(0x42);
		MDMX_DMX_CRC8(0x00);
		MDMX_DMX_CRC8(0xff);
		IIC_DMX_TransmitByte(0x42);
		IIC_DMX_TransmitByte(0x00);
		IIC_DMX_TransmitByte(0xff);
		IIC_DMX_TransmitByte(crc);

		for (f=0x00;f<0xff;f++)
		{
			// Wait forever! I know it is dangerous but I wan't to make sure the byte is received.
			// I want to know if we are hitting wdt so I can fix it....
			IIC_DMX_TransmitByte(level);
		}
		IIC_DMX_TransmitStop();
	}
	//	channel_level=((dmx_channels[f]/level)*0xff);
	//	if (!MDMX_DMX_Update_Channel(0x0,f, channel_level))
	//		MDMX_LCD_Msg_PrintHost(0x0,42);

}
////////////////////////////////////////////////////////////////////////////
// This function returns the absolute vpot value
/////////////////////////////////////////////////////////////////////////////
unsigned char MDMX_DMX_ChannelGet(unsigned char channel)
{
  return dmx_channels[channel];
}

/////////////////////////////////////////////////////////////////////////////
// This function sets the absolute vpot value
/////////////////////////////////////////////////////////////////////////////
void MDMX_DMX_ChannelSet(unsigned char channel, unsigned char value)
{
  dmx_channels[channel] = value;
}


unsigned char MDMX_DMX_CRC8(unsigned char byte)
{
  unsigned char i = (byte ^ crc) & 0xff;
  crc=0;
  if(i & 1)
    crc ^= 0x5e;
  if(i & 2)
    crc ^= 0xbc;
  if(i & 4)
    crc ^= 0x61;
  if(i & 8)
    crc ^= 0xc2;
  if(i & 0x10)
    crc ^= 0x9d;
  if(i & 0x20)
    crc ^= 0x23;
  if(i & 0x40)
    crc ^= 0x46;
  if(i & 0x80)
    crc ^= 0x8c;
  return crc;
}


