// $Id: mdmx_lcd.c 331 2008-05-10 20:31:33Z tk $
/*
 * MIDIbox MM V2 (C version)
 *
 * ==========================================================================
 *
 *  Copyright (C) 2005 Thorsten Klose (tk@midibox.org)
 *  Licensed for personal non-commercial use only.
 *  All other rights reserved.
 * 
 * ==========================================================================
 */

/////////////////////////////////////////////////////////////////////////////
// Include files
/////////////////////////////////////////////////////////////////////////////

#include <cmios.h>

#include "main.h"
#include "mdmx_lcd.h"
#include "mdmx_dmx.h"

/////////////////////////////////////////////////////////////////////////////
// Global Variables
/////////////////////////////////////////////////////////////////////////////

unsigned char lcd_charset;
unsigned char msg_cursor;
unsigned char screen[160];
// the welcome message              <---------------------------------------->
const unsigned char welcome_msg[] = "  ---===< Welcome To MidiboxDMX >===--- ";

// default special character set
const unsigned char charset_vert_bars[8*8] = {
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f, 0x1f,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x1f, 0x1f, 0x1f,
  0x00, 0x00, 0x00, 0x00, 0x1f, 0x1f, 0x1f, 0x1f,
  0x00, 0x00, 0x00, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f,
  0x00, 0x00, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f,
  0x00, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f,
  0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f,
  };

/////////////////////////////////////////////////////////////////////////////
// This function initializes the LCD
/////////////////////////////////////////////////////////////////////////////
void MDMX_LCD_Init(void)
{
  unsigned char i;
	//MIOS_LCD_TypeSet(MIOS_LCD_TYPE_CLCD, 0x80 | 0x37, 0x80 | 0x24);
	;MIOS_LCD_TypeSet(MIOS_LCD_TYPE_CLCD, 0x80 | 0x37, 0x80 | 0x24);
	lcd_charset = 0xff; // force refresh
	MIOS_LCD_CursorSet(0x00);
	MDMX_LCD_SpecialCharsInit(0x00);
	MIOS_LCD_CursorSet(0x80);
	MDMX_LCD_SpecialCharsInit(0x80);
	MIOS_LCD_CursorSet(0x00);

  // initialize msg array
  for(i=0; i<160; ++i) {
    screen[i] = ' ';
  }

  MDMX_LCD_Set_String(0,welcome_msg);
  mdmx_flags.DISPLAY_UPDATE_REQ = 1;

  // request display update
  
}

void MDMX_LCD_Set_String(unsigned char start, unsigned char string[])
{
	unsigned char i;
	for(i=0; string[i] != 0; ++i ) 
	{
		screen[i+start] = string[i];
		msg_cursor++;
	}

}

void MDMX_LCD_Blank(unsigned char line)
{
	// bit0=line1, bit1=line2,bit2=line3,bit3=line3
	unsigned char i;
	if(line & 1)
		for(i=0; i<40; ++i)
			screen[i] = ' ';
	if(line & 2)
		for(i=41; i<80; ++i)
			screen[i] = ' ';
	if(line & 4)
		for(i=81; i<120; ++i)
			screen[i] = ' ';
	if(line & 8)
		for(i=121; i<160; ++i)
			screen[i] = ' ';
}
/////////////////////////////////////////////////////////////////////////////
// This function refreshes the whole LCD screen
/////////////////////////////////////////////////////////////////////////////
void MDMX_LCD_Msg_Refresh(void)
{
  unsigned int i;
  unsigned char saved_cursor;
  // store current cursor in temporary variable
  saved_cursor = msg_cursor;
  //MDMX_LCD_CursorOn();
		    
  // set msg_cursor to first line and update cursor position
  MDMX_LCD_Msg_CursorSet(0x00);
  for(i=0; i<160; ++i) // 1st line of host message
		MDMX_LCD_Msg_PrintChar(screen[i], 1);
 
   for(i=0; i<0x8; ++i)
			MDMX_LCD_PointerPrint(i);
			
  // restore msg_cursor
	MDMX_LCD_CursorOff();
	msg_cursor = saved_cursor;
}

/////////////////////////////////////////////////////////////////////////////
// this function sets the LCD cursor to the msg_cursor position
/////////////////////////////////////////////////////////////////////////////
void MDMX_LCD_Msg_CursorSet(unsigned char cursor_pos)
{
  msg_cursor = cursor_pos;
  // cursor_pos could be mapped to another value here
      // change the cursor if second line has been reached
  if( cursor_pos == 40) 
      MIOS_LCD_CursorSet(0x40); 
  else if( cursor_pos == 80) 
      MIOS_LCD_CursorSet(0x80);
  else if( cursor_pos == 120) 
      MIOS_LCD_CursorSet(0xc0);
  else
	MIOS_LCD_CursorSet(cursor_pos);
}

/////////////////////////////////////////////////////////////////////////////
// this function returns the current cursor value
/////////////////////////////////////////////////////////////////////////////
unsigned char MDMX_LCD_Msg_CursorGet(void)
{
  return msg_cursor;
}

/////////////////////////////////////////////////////////////////////////////
// this function is called when a character should be print to the host message section
/////////////////////////////////////////////////////////////////////////////
void MDMX_LCD_Msg_PrintChar(unsigned char c, unsigned char force)
{
  // print character if not in GPC mode
  // note: with force this check can be bypassed
    // print character
    MIOS_LCD_PrintChar(c);

    // increment cursor
    ++msg_cursor;
  if( msg_cursor ==40 ) 
      MIOS_LCD_CursorSet(0x40); 
  else if( msg_cursor == 80  ) 
      MIOS_LCD_CursorSet(0x80);
  else if( msg_cursor == 120) 
      MIOS_LCD_CursorSet(0xc0);

}

/////////////////////////////////////////////////////////////////////////////
// stores a character in msg_host at a given cursor position and prints it
// if in HOST mode (!gpc)
/////////////////////////////////////////////////////////////////////////////
void MDMX_LCD_Msg_PrintHost(unsigned char cursor, unsigned char c)
{
  screen[cursor] = c;       // save character
  MDMX_LCD_Msg_CursorSet(c);
  MDMX_LCD_Msg_PrintChar(c, 1); // print without force (only print if HOST MSG screen active)
}

/////////////////////////////////////////////////////////////////////////////
// sets a pointer value
/////////////////////////////////////////////////////////////////////////////
void MDMX_LCD_PointerInit(unsigned char type)
{
  //MDMX_VPOT_DisplayTypeSet(type);
  MDMX_LCD_SpecialCharsInit(type);
}

/////////////////////////////////////////////////////////////////////////////
// sets a pointer value
/////////////////////////////////////////////////////////////////////////////
void MDMX_LCD_PointerSet(unsigned char pointer, unsigned char value)
{

  // update screen if pointers are not printed yet
  if( !mdmx_flags.PRINT_POINTERS ) {
    mdmx_flags.PRINT_POINTERS = 1;
    MDMX_LCD_Msg_Refresh();
  } else {
    // print pointer if not in GPC mode
    MDMX_LCD_PointerPrint(pointer);
  }
}

/////////////////////////////////////////////////////////////////////////////
// prints a pointer when not in GPC mode
/////////////////////////////////////////////////////////////////////////////
void MDMX_LCD_PointerPrint(unsigned char pointer)
{
  unsigned char value;


  // get value
  value = MDMX_DMX_ChannelGet(pointer);
  // set cursor
  MIOS_LCD_CursorSet(0xc0 + pointer*5);
  // print value depending on display and pointer type
  if( MIOS_BOX_CFG0.USE_GLCD ) {
    // graphical LCD - here we could add some nice graphic icons
    // currently we just only print a number and a vertical bar...
    //MIOS_LCD_PrintBCD3(value);
	MIOS_LCD_PrintBCD3(value);
    MIOS_LCD_PrintChar((value >> 5) & 0x7f);
    MIOS_LCD_PrintChar(' ');
  } else {
    // character LCD
	MIOS_LCD_PrintBCD3(value);
	MIOS_LCD_PrintChar((value >> 5) & 0x7f);
    MIOS_LCD_PrintChar(' ');
    }
  
}


void MDMX_LCD_WipeOnLR (char *ptr)
{
	// "wipe" on new screen
	char i;
	for (i=0; i<40; i++) {
		MIOS_LCD_CursorSet(0x00+i);
		MIOS_LCD_PrintChar(*(ptr+ 0+i));
		MIOS_LCD_CursorSet(0x40+i);
		MIOS_LCD_PrintChar(*(ptr+40+i));
		MIOS_LCD_CursorSet(0x80+i);
		MIOS_LCD_PrintChar(*(ptr+80+i));
		MIOS_LCD_CursorSet(0xc0+i);
		MIOS_LCD_PrintChar(*(ptr+120+i));
		MIOS_Delay(0x0a);
	}
}

void MDMX_LCD_WipeOnRL (char *ptr)
{
	// "wipe" on new screen
	char i;
	for (i=39; i>=0; i--) {
		MIOS_LCD_CursorSet(0x00+i);
		MIOS_LCD_PrintChar(*(ptr+ 0+i-1));
		MIOS_LCD_CursorSet(0x40+i);
		MIOS_LCD_PrintChar(*(ptr+40+i-1));
		MIOS_LCD_CursorSet(0x80+i);
		MIOS_LCD_PrintChar(*(ptr+80+i-1));
		MIOS_LCD_CursorSet(0xc0+i);
		MIOS_LCD_PrintChar(*(ptr+120+i-1));
		MIOS_Delay(0x0a);
	}
}


void MDMX_LCD_WipeOffRL (void)
{
	// "wipe" off old screen (right to left)
	char i;
	for (i=39; i>=0; i--) {
		#define BLOCK 0xff
		MIOS_LCD_CursorSet(0x00+i);
		MIOS_LCD_PrintChar(BLOCK);
		MIOS_LCD_CursorSet(0x40+i);
		MIOS_LCD_PrintChar(BLOCK);
		MIOS_LCD_CursorSet(0x80+i);
		MIOS_LCD_PrintChar(BLOCK);
		MIOS_LCD_CursorSet(0xc0+i);
		MIOS_LCD_PrintChar(BLOCK);
		MIOS_Delay(0xa);
	}
}


void MDMX_LCD_WipeOffLR (void)
{
	// "wipe" off old screen (left to right)
	char i;
	for (i=0; i<40; i++) {
		#define BLOCK 0xff
		MIOS_LCD_CursorSet(0x00+i);
		MIOS_LCD_PrintChar(BLOCK);
		MIOS_LCD_CursorSet(0x40+i);
		MIOS_LCD_PrintChar(BLOCK);
		MIOS_LCD_CursorSet(0x80+i);
		MIOS_LCD_PrintChar(BLOCK);
		MIOS_LCD_CursorSet(0xc0+i);
		MIOS_LCD_PrintChar(BLOCK);
		MIOS_Delay(0x0a);
	}
}


/////////////////////////////////////////////////////////////////////////////
// initializes special characters
/////////////////////////////////////////////////////////////////////////////
void MDMX_LCD_SpecialCharsInit(unsigned char charset)
{
  // exit if graphical LCD connected
  if( MIOS_BOX_CFG0.USE_GLCD )
    return;

  // exit if charset already initialized
  if( charset == lcd_charset )
    return;

  // else initialize special characters
  lcd_charset = charset;
  switch( charset ) {
     case 0: // "Left justified horizontal bar graph (Aux pots)"
     case 1: // "Centered horizontal Bar graph"
     case 2: // "Right justified horizontal Bar graph"
     case 3: // "Single verticle line (PAN or PAN R)"
     case 4: // "Left justified double verticle line"
     case 5: // "Centered Q spreading bar (Filter Q)"
     case 6: // "Ascending bar graph (Channel Meters)"
     case 7: // "Descending bar graph (Gaon reduction)"
       MIOS_CLCD_SpecialCharsInit(charset_vert_bars);
       break;
     default:
       MIOS_CLCD_SpecialCharsInit(charset_vert_bars);
       break;
  }
}

void MDMX_LCD_CursorOn(void)
{
	MIOS_LCD_Cmd(0x0f); 
}


void MDMX_LCD_CursorOff(void)
{
	MIOS_LCD_Cmd(0x0c); 
}

