/*
 * MIDIbox DMX Controller
 *
 * ==========================================================================
 *
 *  Copyright (C) 2008 Phil Taylor (phil@taylor.org.uk)
 *  Licensed for personal non-commercial use only.
 *  All other rights reserved.
 * 
 * ==========================================================================
 */

/////////////////////////////////////////////////////////////////////////////
// Include files
/////////////////////////////////////////////////////////////////////////////

#include <cmios.h>
#include "main.h"
#include "mdmx_menu.h"
#include "mdmx_lcd.h"

/////////////////////////////////////////////////////////////////////////////
// Global Variables
/////////////////////////////////////////////////////////////////////////////

#define ENTER_MENU 0x80
#define EXIT_MENU 0x40

#define MAIN_MENU_SIZE  4
#define PATCH_MENU_SIZE   2
#define SETUP_MENU_SIZE 2
#define FIXTURE_MENU_SIZE  3
#define ABOUT_MENU_SIZE 2


#define	MENU_BUTTON 0x18
#define EXIT_BUTTON 0x19
#define	LEFT_BUTTON 0x1a
#define RIGHT_BUTTON 0x1b

int menu_selection;
int item_selection;


const int menu_sizes[] = { MAIN_MENU_SIZE-1, PATCH_MENU_SIZE-1, SETUP_MENU_SIZE-1, FIXTURE_MENU_SIZE-1, ABOUT_MENU_SIZE-1 };
enum MDMX_Menu_Function_Index
{
        MDMX_Menu_Patch_Index, MDMX_Menu_Setup_Index, MDMX_Menu_Fixture_Index, 
		MDMX_Menu_Version_Index, MDMX_Menu_ROM_ID_Index, MDMX_Menu_Not_Handled_Index
};
enum MENUS 
{
        MAIN_MENU=0, PATCH_MENU, SETUP_MENU, FIXTURE_MENU, ABOUT_MENU,
        NUMBER_OF_MENUS
};
char current_menu_item[NUMBER_OF_MENUS];
enum ITEMS
{
        SELECT_SETUP_MENU=0,
        SELECT_PATCH_MENU,
        SELECT_FIXTURE_MENU,
        SELECT_ABOUT_MENU,
        PATCH_SET=0,
        PATCH_CLEAR,
        SETUP_DMX=0,
        SETUP_MIDI,
        FIXTURE_NEW,
        FIXTURE_CHANGE,
        FIXTURE_DELETE,
        ABOUT_VERSION=0,
        ABOUT_ROM_ID
};
const unsigned char menu_strings[14][10] = {
        "Main Menu",
        "Setup    ",
        "Patch    ",
        "Fixture  ",
        "About    ",
        "Create   ",
        "Delete   ",
        "DMX      ",
        "MIDI     ",
        "New      ",
        "Change   ",
        "Delete   ",
        "Version  ",
        "ROM ID   "
};
const unsigned char ok_menu_selection_table[] =
{
/*              		 MENU,            		 ITEM,                  						FUNCTION HANDLER, */
            MAIN_MENU,    	SELECT_PATCH_MENU, ENTER_MENU|              PATCH_MENU, 
            MAIN_MENU,  	SELECT_SETUP_MENU, ENTER_MENU|              SETUP_MENU, 
            MAIN_MENU,   	SELECT_FIXTURE_MENU, ENTER_MENU|            FIXTURE_MENU,
            MAIN_MENU,  	SELECT_ABOUT_MENU, ENTER_MENU|              ABOUT_MENU,
            PATCH_MENU,     PATCH_SET,                    	  MDMX_Menu_Patch_Index,
            PATCH_MENU,     PATCH_CLEAR,                      MDMX_Menu_Patch_Index,
            SETUP_MENU,     SETUP_DMX,                        MDMX_Menu_Setup_Index,
            SETUP_MENU,     SETUP_MIDI,                       MDMX_Menu_Setup_Index,
            FIXTURE_MENU,   FIXTURE_NEW,                      MDMX_Menu_Fixture_Index,
            FIXTURE_MENU,   FIXTURE_CHANGE,                   MDMX_Menu_Fixture_Index,
            FIXTURE_MENU,   FIXTURE_DELETE,                   MDMX_Menu_Fixture_Index,
            ABOUT_MENU,     ABOUT_VERSION,                    MDMX_Menu_Version_Index,
            ABOUT_MENU,     ABOUT_ROM_ID,                     MDMX_Menu_ROM_ID_Index
};

const char cancel_menu_selection_table[] =
{
			/*    MENU,            	 ITEM,                  					      FUNCTION HANDLER, */
            MAIN_MENU,  	SELECT_PATCH_MENU,                 MDMX_Menu_Not_Handled_Index,
            MAIN_MENU,  	SELECT_SETUP_MENU,                 MDMX_Menu_Not_Handled_Index,
            MAIN_MENU,   	SELECT_FIXTURE_MENU,               MDMX_Menu_Not_Handled_Index,
            MAIN_MENU,  	SELECT_ABOUT_MENU,                 MDMX_Menu_Not_Handled_Index,
            PATCH_MENU,     PATCH_SET, EXIT_MENU|              MAIN_MENU, 
            PATCH_MENU,     PATCH_CLEAR, EXIT_MENU|            MAIN_MENU,
            SETUP_MENU,     SETUP_DMX, EXIT_MENU|              MAIN_MENU,
            SETUP_MENU,     SETUP_MIDI, EXIT_MENU|             MAIN_MENU,
            FIXTURE_MENU,   FIXTURE_NEW, EXIT_MENU|            MAIN_MENU,
            FIXTURE_MENU,   FIXTURE_DELETE, EXIT_MENU|         MAIN_MENU,
            ABOUT_MENU,     ABOUT_VERSION, EXIT_MENU|          MAIN_MENU, 
            ABOUT_MENU,     ABOUT_ROM_ID, EXIT_MENU|           MAIN_MENU
};


void MDMX_Menu_Display(void)
{
    char i;
    char string_index;
	char cursor_pos=0x40;
	// Blank display
	MDMX_LCD_Blank(0xff);
    string_index=0;
    for(i=0;i<menu_selection;i++)
    {
		string_index+=menu_sizes[i]+2;
    }
	MDMX_LCD_Set_String(0x0,menu_strings[string_index]);
	MDMX_LCD_Msg_CursorSet(30);
	for (i=0;i<=menu_sizes[menu_selection];i++)
	{
		string_index++;
		MDMX_LCD_Set_String(MDMX_LCD_Msg_CursorGet()+10,menu_strings[string_index]);
	}

	//MDMX_LCD_Msg_PrintHost(0x00,0x41);
	mdmx_flags.DISPLAY_UPDATE_REQ = 1;
	mdmx_flags.REFRESH_MENU=0;
}


// Function that handles menu events (MIDI or Keys)
void MDMX_Menu_Event(unsigned char button)
{
	unsigned char cursor_pos=40,i;
	;MDMX_LCD_CursorOn();
    if(button==MENU_BUTTON)
    {
		MDMX_Menu_Request(ok_menu_selection_table);
		mdmx_flags.REFRESH_MENU=1;
    }
    if(button==EXIT_BUTTON)
    {
		MDMX_Menu_Request(cancel_menu_selection_table);
		mdmx_flags.REFRESH_MENU=1;
    }
	
	if (button==LEFT_BUTTON)
	{
		if(item_selection>0)
		{
			item_selection--;
		}
	}
	else if (button==RIGHT_BUTTON)
	{
		if (item_selection<menu_sizes[menu_selection])
		{
			item_selection++;
		}
	}
	
	if (mdmx_flags.REFRESH_MENU)
		MDMX_Menu_Display();
	else
		MDMX_LCD_Msg_CursorSet(40+(item_selection*10));
}


void MDMX_Menu_Selection()
{
	unsigned char i;
	MIOS_LCD_CursorSet(0x40+(item_selection*10));
	for (i=0;i<10;i++)
		MIOS_LCD_PrintChar(0xff);
	
}

const void (* menu_function[])(void) =
{
        MDMX_Menu_Patch, MDMX_Menu_Setup, MDMX_Menu_Fixture, 
		MDMX_Menu_Version, MDMX_Menu_ROM_ID, MDMX_Menu_Not_Handled
};

void MDMX_Menu_Request(char * table_ptr)
{
		enum menu_function_index function_index;
        function_index=0; /* default to first item */
        //#do
        //{
                if( *table_ptr == menu_selection )
                {
                    if( (*table_ptr+1) == item_selection )
                    {
						// This is where we call the function
						//function_index = *(table_ptr+2);
						function_index=0;
                    }
                }
                table_ptr += 3;
        //}
        //while(1);

        if(function_index&ENTER_MENU)
        {
                current_menu_item[menu_selection]=item_selection;
                menu_selection = function_index&(~ENTER_MENU);
                item_selection = current_menu_item[menu_selection];
        }
        else if( function_index&EXIT_MENU )
        {
                current_menu_item[menu_selection]=item_selection;
                menu_selection = function_index&(~EXIT_MENU);
                item_selection = current_menu_item[menu_selection];
        }
        //else
               // (*menu_function[function_index])();
}

void MDMX_Menu_Patch(void)
{
	return;
}

void MDMX_Menu_Setup(void)
{
	return;
}

void MDMX_Menu_Fixture(void) 
{
	return;
}
void MDMX_Menu_Version(void)
{
	return;
}

void MDMX_Menu_ROM_ID(void)
{
	return;
}
void MDMX_Menu_Not_Handled(void)
{
	// This function is for error conditions if the menu doesn't exist for any reason.
}

