// $Id: mm_midi.c 331 2008-05-10 20:31:33Z tk $
/*
 * MIDIbox MM V2 (C version)
 *
 * ==========================================================================
 *
 *  Copyright (C) 2005 Thorsten Klose (tk@midibox.org)
 *  Licensed for personal non-commercial use only.
 *  All other rights reserved.
 * 
 * ==========================================================================
 */

/////////////////////////////////////////////////////////////////////////////
// Include files
/////////////////////////////////////////////////////////////////////////////

#include <cmios.h>

#include "main.h"
#include "mdmx_midi.h"
#include "mdmx_dmx.h"
#include "mdmx_lcd.h"
#include "mdmx_menu.h"

/////////////////////////////////////////////////////////////////////////////
// Global Variables
/////////////////////////////////////////////////////////////////////////////

unsigned char master_fader;

/////////////////////////////////////////////////////////////////////////////
// This function is called from MPROC_NotifyReceivedEvnt when a complete
// MIDI event has been received
/////////////////////////////////////////////////////////////////////////////
void MDMX_MIDI_Received(unsigned char evnt0, unsigned char evnt1, unsigned char evnt2)
{
  static unsigned char fader;
  static unsigned char fader_pos_l;
  static unsigned char fader_pos_h;
  static unsigned char led_id_l;
  static unsigned char led_id_h;

  // exit if wrong channel
  if( (evnt0 & 0x0f) != ((MDMX_MIDI_CHANNEL-1) & 0x0f) )
    return;

  // extract MIDI status (without channel)
  evnt0 &= 0xf0;

  // for easier parsing: convert Note Off to Note On with velocity 0
  if( (evnt0 & 0xf0) == 0x90 ) {
    evnt0 |= 0x10;
    evnt2 = 0x00;
  }

  if( evnt0 == 0x90 ) { // Note On/Off

    // ping: if 90 00 00 has been received, return 90 00 7F
    if( evnt1 == 0x00 && evnt2 == 0x00 ) 
	{
      MIOS_MIDI_BeginStream();
      MIOS_MIDI_TxBufferPut(0x90 | ((MDMX_MIDI_CHANNEL-1) & 0x0f));
      MIOS_MIDI_TxBufferPut(0x00);
      MIOS_MIDI_TxBufferPut(0x7f);
      MIOS_MIDI_EndStream();
    }
	
	//if (evnt2 !=0x00)// Note on for remote control commands
		MDMX_Menu_Event(evnt1);

  } else if( evnt0 == 0xb0) { // CC

    if( (evnt1 & 0x80) == 0x00 ) {        // Bn 0[0-16]: Fader MSB
      fader_pos_h = evnt2;
    //} else if( (evnt1 & 0x78) == 0x20 ) { // Bn 2[0-7]: Fader LSB
      //fader_pos_l = evnt2;

	 // position = ((unsigned int)position_h << (7-4)) | ((unsigned int)position_l >> 4);
	  if (evnt1 == 0x07){
			MDMX_DMX_Update_Master(fader_pos_h << 1 );
	  }else {
			if (!MDMX_DMX_Update_Channel(0x0, evnt1, fader_pos_h << 1)) // Convert to 8 bit resolution
				MDMX_LCD_Msg_PrintHost(0x0,42);

		}
	  //MDMX_MF_FaderMove(evnt1 & 0x07, fader_pos_l, fader_pos_h);
    } else if( evnt1 == 0x0c ) {          // Bn 0C: LED ID_H
      led_id_h = evnt2;
    } else if( evnt1 == 0x2c ) {          // Bn 2C: LED ID_L
      led_id_l = evnt2;
     // MDMX_DIO_LEDHandler(led_id_l, led_id_h);
    }

  }
  
 }
